/**
    @file ft51_usb_finalise.c

    @brief USB Library API finalise function.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */
#include <stdlib.h>
#include <string.h>
#include "ft51.h"
#include "registers/ft51_registers.h"
#include "ft51_usb.h"
#include "ft51_usb_internal.h"

void USB_finalise(void)
{
    DISABLE_INTERRUPTS;
    // Pull D+ high to make a disconnect condition on the bus.
    FT122_CMD = SET_MODE;
    FT122_DATA = 0x00;  // Disable D+ pullup
    FT122_DATA = 0x43;  // CLOCKOUT = 48 MHz / (3 + 1) = 12 MHz

    // Disable all endpoints (except control endpoints).
    FT122_CMD = SET_ENDPOINT_ENABLE;
    FT122_DATA = 0x00;

    // Note: Endpoints are disabled in the bus reset handler and address
    // is set to zero when host resets device after USB is reenabled.
    USB_current_state = ATTACHED;
    ENABLE_INTERRUPTS;
}